#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`exceptions`
==================

.. module:: exceptions
   :platform: Unix, Windows
   :synopsis:

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2014-09-10, 08:29

"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import

import six


def get_error_class(exc, exception_text):
    error_class = _ERROR_CODE_TO_CLASS.get(six.text_type(exc.fault.faultstring))
    if error_class is None:
        return BankIDError("{0}: {1}".format(exc, exception_text))
    else:
        return error_class(exception_text)


class BankIDError(Exception):
    """Parent exception class for all PyBankID errors."""

    def __init__(self, *args, **kwargs):
        super(BankIDError, self).__init__(*args, **kwargs)
        self.rfa = None


class BankIDWarning(Warning):
    """Warning class for PyBankID."""
    pass


class InvalidParametersError(BankIDError):
    """User induced error.

    Code: INVALID_PARAMETERS
    Reason: Invalid parameter. Invalid use of method.
    Action by RP: RP must not try the same request again. This is
                  an internal error within RP's system and must
                  not be communicated to the user as a BankID-error.

    """


class AlreadyInProgressError(BankIDError):
    """Failure to create new order due to one already in progress.

    Code: ALREADY_IN_PROGRESS
    Reason: An order for this user is already in progress. The order
            is aborted. No order is created.
    Action by RP: RP must inform the user that a login or signing
                  operation is already initiated for this user.
                  Message RFA3 should be used.

    """

    def __init__(self, *args, **kwargs):
        super(AlreadyInProgressError, self).__init__(*args, **kwargs)
        self.rfa = 3


class InternalError(BankIDError):
    """Remote server error.

    Code: INTERNAL_ERROR
    Reason: Internal technical error in the BankID system.
    Action by RP: RP must not automatically try again. RP must
                  inform the user that a technical error has
                  occurred. Message RFA5 should be used.

    """

    def __init__(self, *args, **kwargs):
        super(InternalError, self).__init__(*args, **kwargs)
        self.rfa = 5


class RetryError(BankIDError):
    """Remote server error, different from InternalError.

    Code: RETRY
    Reason: Internal technical error in the BankID system.
    Action by RP: RP must not automatically try again. RP must
                  inform the user that a technical error has
                  occurred. Message RFA5 should be used.

    """

    def __init__(self, *args, **kwargs):
        super(RetryError, self).__init__(*args, **kwargs)
        self.rfa = 5


class AccessDeniedRPError(BankIDError):
    """Access permission denied error.

    Code: ACCESS_DENIED_RP
    Reason: RP does not have access to the service or requested
            operation.
    Action by RP: RP must not try the same request again. This is
                  an internal error within RP's system and must
                  not be communicated to the user as a BankID-error.

    """


class ClientError(BankIDError):
    """Remote technical error.

    Code: CLIENT_ERR
    Reason: Internal technical error. It was not possible to
            create or verify the transaction.
    Action by RP: RP must not automatically try again. RP must
                  inform the user. Message RFA12.

    """

    def __init__(self, *args, **kwargs):
        super(ClientError, self).__init__(*args, **kwargs)
        self.rfa = 12


class ExpiredTransactionError(BankIDError):
    """Error due to collecting on an expired order.

    Code: EXPIRED_TRANSACTION
    Reason: The order has expired. The BankID security
            app/program did not start, the user did not
            finalize the signing or the RP called collect
            too late.
    Action by RP: RP must inform the user. Message RFA8.

    """

    def __init__(self, *args, **kwargs):
        super(ExpiredTransactionError, self).__init__(*args, **kwargs)
        self.rfa = 8


class CertificateError(BankIDError):
    """Error due to certificate issues.

    Code: CERTIFICATE_ERR
    Reason: This error is returned if:
                1) The user has entered wrong security code
                   too many times in her mobile device. The
                   Mobile BankID cannot be used.
                2) The users BankID is revoked.
                3) The users BankID is invalid.
    Action by RP: RP must inform the user. Message RFA3.

    """

    def __init__(self, *args, **kwargs):
        super(CertificateError, self).__init__(*args, **kwargs)
        self.rfa = 3


class UserCancelError(BankIDError):
    """User had issue a cancel on the order.

    Code: USER_CANCEL
    Reason: The user decided to cancel the order.
    Action by RP: RP must inform the user. Message RFA6.

    """

    def __init__(self, *args, **kwargs):
        super(UserCancelError, self).__init__(*args, **kwargs)
        self.rfa = 6


class CancelledError(BankIDError):
    """User had issue a cancel on the order.

    Code: CANCELLED
    Reason: The order was cancelled. The system
            received a new order for the user.
    Action by RP: RP must inform the user. Message RFA3.

    """

    def __init__(self, *args, **kwargs):
        super(CancelledError, self).__init__(*args, **kwargs)
        self.rfa = 3


class StartFailedError(BankIDError):
    """Error handling the order's progression due to RP/user issues.

    Code: START_FAILED
    Reason: The user did not provide her ID, or the RP
            requires autostarttoken to be used, but the
            client did not start within a certain time limit.
            The reason may be::
                1) RP did not use autoStartToken when
                   starting BankID security program/app.
                2) The client software was not installed or
                   other problem with the user’s computer.
    Action by RP: 1) The RP must use autoStartToken when
                     starting the client.
                  2) The RP must inform the user. Message
                     RFA17.

    """

    def __init__(self, *args, **kwargs):
        super(StartFailedError, self).__init__(*args, **kwargs)
        # TODO: Dual cause, in which only one requires RFA. Remove?
        self.rfa = 17


_ERROR_CODE_TO_CLASS = {
    'INVALID_PARAMETERS': InvalidParametersError,
    'ALREADY_IN_PROGRESS': AlreadyInProgressError,
    'INTERNAL_ERROR': InternalError,
    'RETRY': RetryError,
    'ACCESS_DENIED_RP': AccessDeniedRPError,
    'CLIENT_ERR': ClientError,
    'EXPIRED_TRANSACTION': ExpiredTransactionError,
    'CERTIFICATE_ERR': CertificateError,
    'USER_CANCEL': UserCancelError,
    'CANCELLED': CancelledError,
    'START_FAILED': StartFailedError,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`bankid`
==================

.. module:: bankid
   :platform: Unix, Windows
   :synopsis:

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2014-09-09, 16:55

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import os
import tempfile
import subprocess
import sys
import requests

_TEST_CERT_PASSWORD = 'qwerty123'
_TEST_CERT_URL = "https://www.bankid.com/assets/bankid/rp/FPTestcert2_20150818_102329.pfx"


def create_bankid_test_server_cert_and_key(destination_path):
    """Fetch the P12 certificate from BankID servers, split it into
    a certificate part and a key part and save them as separate files,
    stored in PEM format.

    :param destination_path: The directory to save certificate and key files to.
    :type destination_path: unicode
    :returns: The path tuple ``(cert_path, key_path)``.
    :rtype: tuple

    """
    if sys.platform == 'win32':
        raise NotImplementedError(
            "Test certificate fetching in Windows not supported. "
            "See documentation for details.")

    certificate, key = split_test_cert_and_key()

    # Paths to output files.
    out_cert_path = os.path.join(os.path.abspath(
        os.path.expanduser(destination_path)), 'cert.pem')
    out_key_path = os.path.join(os.path.abspath(
        os.path.expanduser(destination_path)), 'key.pem')

    with open(out_cert_path, 'wt') as f:
        f.write(certificate)
    with open(out_key_path, 'wt') as f:
        f.write(key)

    # Return path tuples.
    return out_cert_path, out_key_path


def split_test_cert_and_key():
    """Fetch the P12 certificate from BankID servers, split it into
    a certificate part and a key part and return the two components as text data.

    :returns: Tuple of certificate and key string data.
    :rtype: tuple

    """
    # Paths to temporary files.
    cert_tmp_path = os.path.join(tempfile.gettempdir(), os.path.basename(_TEST_CERT_URL))
    cert_conv_tmp_path = os.path.join(tempfile.gettempdir(), 'certificate.pem')
    key_conv_tmp_path = os.path.join(tempfile.gettempdir(), 'key.pem')

    # Fetch P12 certificate and store in temporary folder.
    r = requests.get(_TEST_CERT_URL)
    with open(cert_tmp_path, 'wb') as f:
        f.write(r.content)

    # Use openssl for converting to pem format.
    pipeline_1 = [
        'openssl', 'pkcs12',
        '-in', "{0}".format(cert_tmp_path),
        '-passin', 'pass:{0}'.format(_TEST_CERT_PASSWORD),
        '-out', "{0}".format(cert_conv_tmp_path),
        '-clcerts', '-nokeys'
    ]
    p = subprocess.Popen(pipeline_1,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    p.communicate()
    pipeline_2 = [
        'openssl', 'pkcs12',
        '-in', "{0}".format(cert_tmp_path),
        '-passin', 'pass:{0}'.format(_TEST_CERT_PASSWORD),
        '-out', "{0}".format(key_conv_tmp_path),
        '-nocerts', '-nodes'
    ]
    p = subprocess.Popen(pipeline_2,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    p.communicate()

    # Open the newly created PEM certificate in the temporary folder.
    with open(cert_conv_tmp_path, 'rt') as f:
        certificate = f.read()
    with open(key_conv_tmp_path, 'rt') as f:
        key = f.read()

    # Try to remove all temporary files.
    try:
        os.remove(cert_tmp_path)
        os.remove(cert_conv_tmp_path)
        os.remove(key_conv_tmp_path)
    except:
        pass

    return certificate, key


def main():
    paths = create_bankid_test_server_cert_and_key(os.path.expanduser('~'))
    print('Saved certificate as {0}'.format(paths[0]))
    print('Saved key as {0}'.format(paths[1]))

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Release data for the PyBankID project."""

# A quick and dirty fix for installations on Heroku.
try:
    from .client import BankIDClient
    import bankid.exceptions as exceptions
    from .testcert import create_bankid_test_server_cert_and_key

    __all__ = ['BankIDClient', 'exceptions', 'create_bankid_test_server_cert_and_key', 'version']
except ImportError:
    pass

# Name of the package for release purposes.  This is the name which labels
# the tarballs and RPMs made by distutils, so it's best to lowercase it.
_name = 'pybankid'

# Version information.  An empty _version_extra corresponds to a full
# release.  'dev' as a _version_extra string means this is a development
# version.
_version_major = 0
_version_minor = 3
_version_patch = 3
# _version_extra = 'dev2'
# _version_extra = 'a1'
_version_extra = ''  # Uncomment this for full releases

# Construct full version string from these.
_ver = [_version_major, _version_minor, _version_patch]

__version__ = '.'.join(map(str, _ver))
if _version_extra:
    __version__ = __version__ + '.' + str(_version_extra)

version = __version__  # backwards compatibility name
version_info = (_version_major, _version_minor, _version_patch, _version_extra)

__description__ = "BankID client for Python"

__license__ = 'MIT'

__authors__ = {
    'hbldh': ('Henrik Blidh', 'henrik.blidh@nedomkull.com'),
}
__author__ = 'Henrik Blidh'
__author_email__ = 'henrik.blidh@nedomkull.com'
__url__ = 'https://github.com/hbldh/pybankid/'
__download_url__ = 'https://github.com/hbldh/pybankid/tarball/' + '.'.join(map(str, _ver))

__platforms__ = ['Linux', 'Mac OSX', 'Windows XP/Vista/7/8']
__keywords__ = ['BankID', 'SOAP']
__classifiers__ = [
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'License :: OSI Approved :: MIT License',
    'Operating System :: POSIX :: Linux',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: MacOS :: MacOS X',
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Security',
    'Topic :: Utilities',
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:mod:`bankid`
==================

.. module:: bankid
   :platform: Unix, Windows
   :synopsis:

.. moduleauthor:: hbldh <henrik.blidh@nedomkull.com>

Created on 2014-09-09, 16:55

"""

from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import

import os
import warnings
import six
import base64
import datetime

import requests
from suds.client import Client
from suds.transport.http import HttpAuthenticated
from suds.transport import Reply
from suds import WebFault
from suds.sax.text import Text
from pkg_resources import resource_filename

from bankid.exceptions import get_error_class, BankIDWarning

# Handling Python 2.7 verification of certificates with urllib3.
# See README.rst for details.
try:
    import requests.packages.urllib3.contrib.pyopenssl
    requests.packages.urllib3.contrib.pyopenssl.inject_into_urllib3()
except ImportError:
    if bool(os.environ.get('PYBANKID_DISABLE_WARNINGS', False)):
        requests.packages.urllib3.disable_warnings()


class BankIDClient(object):

    def __init__(self, certificates, test_server=False):
        self.certs = certificates

        if test_server:
            self.api_url = 'https://appapi.test.bankid.com/rp/v4'
            self.wsdl_url = 'https://appapi.test.bankid.com/rp/v4?wsdl'
            self.verify_cert = resource_filename('bankid.certs', 'appapi.test.bankid.com.pem')
        else:
            self.api_url = 'https://appapi.bankid.com/rp/v4'
            self.wsdl_url = 'https://appapi.bankid.com/rp/v4?wsdl'
            self.verify_cert = resource_filename('bankid.certs', 'appapi.bankid.com.pem')

        headers = {
            "Content-Type": "text/xml;charset=UTF-8",
            "SOAPAction": ""
        }
        t = RequestsTransport(cert=self.certs, verify_cert=self.verify_cert)
        self.client = Client(self.wsdl_url, location=self.api_url,
                             headers=headers, transport=t)

    def authenticate(self, personal_number, **kwargs):
        """Request an authentication order. The :py:mod:`collect` method
        is used to query the status of the order.

        :param personal_number: The Swedish personal number in format YYYYMMDDXXXX.
        :type personal_number: unicode
        :return: The OrderResponse parsed to a dictionary.
        :rtype: dict
        :raises BankIDError: raises a subclass of this error
                             when error has been returned from server.

        """
        if 'requirementAlternatives' in kwargs:
            warnings.warn("Requirement Alternatives option is not tested.", BankIDWarning)

        try:
            out = self.client.service.Authenticate(
                personalNumber=personal_number, **kwargs)
        except WebFault as e:
            raise get_error_class(e, "Could not complete Authenticate order.")

        return self._dictify(out)

    def sign(self, user_visible_data, personal_number=None, **kwargs):
        """Request an signing order. The :py:mod:`collect` method
        is used to query the status of the order.

        :param user_visible_data: The information that the end user is requested to sign.
        :type user_visible_data: unicode
        :param personal_number: The Swedish personal number in format YYYYMMDDXXXX.
        :type personal_number: unicode
        :return: The OrderResponse parsed to a dictionary.
        :rtype: dict
        :raises BankIDError: raises a subclass of this error
                     when error has been returned from server.

        """
        if 'requirementAlternatives' in kwargs:
            warnings.warn("Requirement Alternatives option is not tested.", BankIDWarning)

        try:
            out = self.client.service.Sign(
                userVisibleData=base64.b64encode(user_visible_data),
                personalNumber=personal_number, **kwargs)
        except WebFault as e:
            raise get_error_class(e, "Could not complete Sign order.")

        return self._dictify(out)

    def collect(self, order_ref):
        """Collect the progress status of the order with the specified
        order reference.

        :param order_ref: The UUID string specifying which order to collect status from.
        :type order_ref: unicode
        :return: The CollectResponse parsed to a dictionary.
        :rtype: dict
        :raises BankIDError: raises a subclass of this error
                             when error has been returned from server.

        """
        try:
            out = self.client.service.Collect(orderRef=order_ref)
        except WebFault as e:
            raise get_error_class(e, "Could not complete Collect call.")

        return self._dictify(out)

    def file_sign(self, **kwargs):
        """Request a file signing order. The :py:mod:`collect` method
        is used to query the status of the order.

        .. note::

            Not implemented due to that the method is deprecated.

        """
        raise NotImplementedError(
            "FileSign is deprecated and therefore not implemented.")

    def _dictify(self, doc):
        """Transforms the replies from :py:mod:`suds` own types to a
        regular Python dict with strings and datetimes.

        Tested with BankID version 2.5 return data.

        :param doc: The response as interpreted by :py:mod:`suds`.
        :returns: The response parsed to a dict.
        :rtype: dict

        """
        doc = dict(doc)
        out = {}
        try:
            for k in doc:
                k = six.text_type(k)
                if isinstance(doc[k], Text):
                    out[k] = six.text_type(doc[k])
                elif isinstance(doc[k], datetime.datetime):
                    out[k] = doc[k]
                else:
                    out[k] = self._dictify(doc[k])
        except:
            out = doc

        return out


class RequestsTransport(HttpAuthenticated):
    """A Requests-based transport for suds, enabling the use of https and
    certificates when communicating with the SOAP service.

    Code has been adapted from
    http://stackoverflow.com/questions/6277027/suds-over-https-with-cert

    """
    def __init__(self, **kwargs):
        self.requests_session = requests.Session()
        self.requests_session.cert = kwargs.pop('cert', None)
        self.requests_session.verify = kwargs.pop('verify_cert', None)
        # `super` won't work because HttpAuthenticated does not use new style class.
        HttpAuthenticated.__init__(self, **kwargs)

    def open(self, request):
        """Fetches the WSDL specification using certificates."""
        self.addcredentials(request)
        resp = self.requests_session.get(request.url,
                                         data=request.message,
                                         headers=request.headers)
        result = six.BytesIO(six.b(resp.content.decode('utf-8')))
        return result

    def send(self, request):
        """Posts to SOAP service using certificates."""
        self.addcredentials(request)
        resp = self.requests_session.post(request.url,
                                          data=request.message,
                                          headers=request.headers)
        result = Reply(resp.status_code, resp.headers, resp.content)
        return result



# -*- coding: utf-8 -*-
"""
PyBankID
========

PyBankID is a client for performing BankID signing.

The Swedish BankID solution for digital signing uses a SOAP
connection solution, and this module aims at providing a simplifying
client for making authentication, signing and collect requests to
the BankID servers.

The latest development version is available at the project's `GitHub
site <https://github.com/hbldh/pybankid/>`_.

Created by hbldh <henrik.blidh@nedomkull.com>
Created on 2013-09-14, 19:31

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import os
from setuptools import setup, find_packages
import bankid

# Get the long description from the README file
try:
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.rst')) as f:
        long_description = f.read()
except:
    long_description = __doc__


setup(
    name='pybankid',
    version=bankid.__version__,
    author=bankid.__author__,
    author_email=bankid.__author_email__,
    description=bankid.__description__,
    long_description=long_description,
    license=bankid.__license__,
    url=bankid.__url__,
    classifiers=bankid.__classifiers__,
    platforms=bankid.__platforms__,
    packages=find_packages(exclude=('tests', )),
    package_data={'': ['*.pem']},
    install_requires=[
        'requests>=2.7.0',
        'suds-jurko>=0.6',
        'six>=1.9.0'
    ],
    dependency_links=[],
    ext_modules=[],
    entry_points={
    }
)
